#! /usr/bin/env node

/**
 * Basic CLI for testing
 *
 * Usage:
 *  <this> input_file [-l]
 *
 *  input_file  - File to process
 *  -l          - Generate line control statements
 *
 * @author Mikhail Yurasov <me@yurasov.me>
 */

'use strict';

const Builder = require('./index');
const packageJson = require('../package.json');
const path = require('path');

/**
 * Blackhole logger
 */
class NullLogger {
  debug() {
  }

  info() {
  }

  warning() {
  }

  error() {
  }
}

if (/* check if we have enought args */ process.argv.length < 3 || process.argv.length > 4) {

  // print usage info
  console.log(
    `
\u001b[36m${packageJson.name}\u001b[39m v${packageJson.version}
usage: ${Object.getOwnPropertyNames((packageJson.bin))[0]} <input_file> [options]
-l - generate line control statements
    `.trim());

} else {

  const builder = new Builder();
  builder.logger = new NullLogger();

  let inputFile;

  // set options
  for (let i = 2; i < process.argv.length; i++) {
    if ('-l' === process.argv[i]) {
      builder.machine.generateLineControlStatements = true;
    } else {
      inputFile = process.argv[i];
    }
  }

  // set the directory of the input file as first search dir
  builder.machine.readers.file.searchDirs.unshift(path.dirname(path.resolve(inputFile)));

  // execute
  const res = builder.machine.execute(`@include "${inputFile.replace(/\"/g, `'`)}"`);

  // here we go
  console.log(res);
}
